def boxify(header, params):
    lines = []

    max_key_width = max(len(header), max(len(str(key)) for key in params.keys()))
    max_value_width = max(len(str(key)) for key in params.values())
    width = max_key_width + max_value_width

    # Add headline
    lines.append(f"\n\t+{'-' * (width + 4)}+")
    lines.append(f"| \033[1;36;10m{header:^{width + 2}}\033[0m |")
    lines.append(f"+{'-' * (width + 4)}+")

    # Add parameters
    for key, value in params.items():
        lines.append(f"| \033[1;38;10m{key :<{max_key_width}}\033[0m : \033[1;31;10m{str(value):<{max_value_width}}\033[0m|")
    lines.append(f"+{'-' * (width+4)}+")

    box = "\n\t".join(lines)
    return box



import logging
import os
import sys
from logging.handlers import TimedRotatingFileHandler
from typing import Any, Union

from .CustomFormatter import CoolFormatter

# Define the default logging level
DEFAULT_LOGGING_LEVEL = logging.INFO

# Define the default log directory
DEFAULT_LOG_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'logs')


class LoggerSingleton:
    __instance: 'LoggerSingleton' = None
    __log_level: int = DEFAULT_LOGGING_LEVEL
    __log_dir: str = DEFAULT_LOG_DIR

    @classmethod
    def get_instance(cls) -> 'LoggerSingleton':
        if cls.__instance is None:
            cls.__instance = cls()
        return cls.__instance

    def __init__(self) -> None:
        self.loggers = {}

    # New setter method for changing log level
    def set_level(self, level: int) -> None:
        self.__log_level = level

    # New setter method for changing log directory
    def set_log_dir(self, dir_path: str) -> None:
        self.__log_dir = os.path.join(dir_path, 'logs')
        if not os.path.exists(self.__log_dir):
            os.makedirs(self.__log_dir)

    def get_console_handler(self) -> logging.StreamHandler:
        console_handler = logging.StreamHandler(sys.stdout)
        console_handler.setFormatter(self.get_formatter())
        return console_handler

    def get_file_handler(self) -> TimedRotatingFileHandler:
        file_handler = TimedRotatingFileHandler(os.path.join(self.__log_dir, 'logs.log'), when='midnight')
        file_handler.setFormatter(self.get_file_formatter())
        return file_handler

    def get_formatter(self) -> CoolFormatter:
        return CoolFormatter()

    def get_file_formatter(self) -> logging.Formatter:
        return logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")

    def get_logger(self, logger_name: str) -> logging.Logger:
        if logger_name in self.loggers:
            return self.loggers[logger_name]

        logger = logging.getLogger(logger_name)
        logger.setLevel(self.__log_level)  # Use the current log level
        logger.addHandler(self.get_console_handler())
        logger.addHandler(self.get_file_handler())
        logger.propagate = False

        self.loggers[logger_name] = logger
        return logger


def get_logger(logger_name: str) -> logging.Logger:
    return LoggerSingleton.get_instance().get_logger(logger_name)


# New utility methods
def set_log_level(level: int) -> None:
    LoggerSingleton.get_instance().set_level(level)


def set_log_dir(dir_path: str) -> None:
    LoggerSingleton.get_instance().set_log_dir(dir_path)



from setuptools import setup

setup(
    name='uh_custom_logger',
    version='0.0.2',
    description='A custom logging utility for uh projects.',
    author='Necip Asik',
    author_email='data@universityhousing.nl',
    packages=['uh_custom_logger'],  # same as name
)



import logging
import time

class CoolFormatter(logging.Formatter):
    # ANSI color codes
    COLOR_CODES = {
        "MAGENTA": "\033[95m",
        "BLUE": "\033[94m",
        "GREEN": "\033[92m",
        "YELLOW": "\033[93m",
        "RED": "\033[91m",
        "BOLD": "\033[1m",
        "UNDERLINE": "\033[4m",
        "END": "\033[0m",
        "GREY": "\033[37m"
    }

    # Mapping from logging levels to colors
    LEVEL_TO_COLOR = {
        "DEBUG": COLOR_CODES["BLUE"],
        "INFO": COLOR_CODES["GREEN"],
        "WARNING": COLOR_CODES["YELLOW"],
        "ERROR": COLOR_CODES["RED"],
        "CRITICAL": COLOR_CODES["BOLD"] + COLOR_CODES["RED"]
    }

    def format(self, record):
        level_name = record.levelname
        colored_level_name = self.LEVEL_TO_COLOR.get(level_name, "") + level_name + self.COLOR_CODES["END"]
        colored_name = self.COLOR_CODES["MAGENTA"] + record.name + self.COLOR_CODES["END"]

        message = super().format(record)
        timestamp = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(record.created))

        return f"{self.COLOR_CODES['GREY']}{timestamp}{self.COLOR_CODES['END']} - {colored_level_name} - {colored_name} - {message}"
